// $Id: httpstats.C,v 1.1 2001/04/16 17:43:06 dvermeir Exp $

#include <string>
#include <fstream>
#include <strstream>

// Httpstats main function.

static const string usage("httpstats configfile logfile");

#include "configuration.h"
#include "logrecord.h"
#include "stats.h"

// Default configuration information.
const string default_config_text(
      "format path 1-4\nselect path /\nexclude path /robots.txt\n");

int 
main(int argc,char* argv[]) {
// We put the lot in a try .. catch block, in case something throws
// an exception, which should not happen but..
try {
  // 1. check arguments: there must be 1 or 2 of them
  if (argc<2 || argc>3) {
    cerr << "Usage: " << usage << endl;
    return 1; 
    }
  
  string log_file_name(argv[1]);

  // 2. parse configuration file. We will use a class Configuration that
  // to contain the configuration information. 

  istream* config_file(0);
  string config_file_name((argc==3) ? argv[2] : "default");

  if (argc==3) {
    // we have a configuration file argument

    config_file = new ifstream(config_file_name.c_str());

    if (!*config_file) {
      cerr << "Cannot open configuration file \"" 
           << config_file_name << "\"" << endl;
      return 1;
      }
    }
  else 
    config_file = new istrstream(default_config_text.data(),
                                 default_config_text.size());
    
  
  // Parse configuration.
  Configuration config; 
  unsigned int err_lineno(config.parse(*config_file));
  if (err_lineno) {
    cerr << "Error on line " << err_lineno
         << " in configuration file \"" << config_file_name << "\"" << endl;
    return 1;
    }
  
  delete config_file; config_file = 0; // not needed anymore

  cerr << config << endl;
  // 3. Process the log file line by line and process each line using config.
  ifstream log_file(log_file_name.c_str());
  if (!log_file) {
    cerr << "Cannot open log file \"" << log_file_name << "\"" << endl;
    return 1;
    }
  
  Stats             stats(config);  // A stats object must know about config.
  const Expression* select(config.select()); // Records must satisfy this to get into stats.
  string            line; //  Buffer for lines from the log_file.
  LogRecord         logrecord; // A line will be parsed into this object.
  int               n_lines_parsed(0); // Number of lines succesfully parsed.
  int               n_lines_selected(0); // Number of lines parsed and selected.
  int               n_error_lines(0); // Number of lines with errors.
  
  // Processing log file..

  while (getline(log_file,line)) {
    if (logrecord.parse_line(line,config)) {
      if (select) {
        if (select->eval(logrecord)) {
          ++n_lines_selected;
          stats.add(logrecord);
          }
        }
      else {
        stats.add(logrecord);
        ++n_lines_selected;
        }
      ++n_lines_parsed;
      }
    else
      ++n_error_lines;
    }
  
  cerr << "// Generate statistics output." << endl;
  cout << stats;
  
  // Show something on cerr to keep the user amused.
  cerr << n_lines_parsed << " lines processed, " 
       << n_lines_selected << " lines selected, " 
       << n_error_lines << " lines had errors" << endl;
  }
catch (exception& e) {
  cerr << e.what() << endl;
  }
}

