// $Id: path.C,v 1.1 2001/04/16 17:43:06 dvermeir Exp $
#include "path.h"

// Construct path containing the first n components of p.
Path::Path(const Path& p,unsigned int n) {
unsigned int m(min(p.size(),n));
path_.resize(m);
for (unsigned int i=0; i<m; ++i) 
  path_[i] = p[i];
}

bool
Path::parse(const string& ps) {
str_.erase();
path_.clear();
/* Decode the parts of the ps into a vector. The only component of the
   vector will be empty iff the ps  == "/". 
   We also refuse to append empty strings to path_. This handles cases like
   "/a//b" which will result in <"a","b">.

  We use n0 to indicate the start position of a component and
  n1 to indicate the position just after the end of the component.
*/
string::size_type n0(ps.find_first_not_of('/',0));

if (n0==string::npos) { // ps must be "/"
  path_.push_back("");
  return true;
  }

string::size_type n1(string::npos);

/* The variable n0 points to the start of a component. We make n1 point to
   the next '/', if any. The component then is the substring starting at
   n0 and ending before n1.
*/
while ((n1=ps.find_first_of('/',n0))!=string::npos) {
  if (n1>n0) // only non-empty parts are stored.
    path_.push_back(ps.substr(n0,n1-n0));
  n0 = n1+1;
  }
// If ps does not end with '/', we must still add the final component.
if (n0<ps.size())
  path_.push_back(ps.substr(n0));

return true;
}

const string&
Path::str() const {
if (str_.size()==0) {
  for (unsigned int j=0; j<size() ; ++j) {
    str_ += '/';
    str_ += path_[j];
    }
  }
return str_;
}

ostream& 
operator<<(ostream& os,const Path& path) {
return os << path.str();
}

